import ASE.Loader_impl as implMod

class Loader:
  '''
  Class <code>Loader</code> manages dyanamic loading and symbol name resolution for the SIDL runtime system
  - This class keeps a record of all libraries loaded through this interface, including the initial "global" symbols in the main program
  IMPL: When you start up, create a bunch of DLL objects with the classes specified, but do not load them. When
  IMPL: resolving a symbol, maintain an internal map of names to DLL objects. Then check if the library is loaded, and if not load it.
  IMPL: If the class is not found in our map, then do the search path thing.
  '''
  
  def __init__(self, obj = None, subClasses = None):
    '''
    A Bootstrap class wraps an implementation class (or another bootstrap class). The subclass list is used for virtualizing calls.
    We have to initialize the parent first in case the implementation object makes calls during __init__()
    '''
    self.classes = []
    if obj is None:
      if not subClasses is None:
        self.classes.extend(subClasses)
      import ASE.BaseClass
      self.parent = ASE.BaseClass.BaseClass(subClasses = self.classes)
      self.obj = implMod.Loader(self)
      self.classes.append(self.obj)
      self.parent.classes.insert(-1, self.obj)
    else:
      self.obj = obj
      self.parent = obj
  
  def __getattr__(self, attr):
    for klass in self.classes:
      if hasattr(klass, attr):
        return getattr(klass, attr)
    if not self.parent is None:
      return getattr(self.parent, attr)
    raise AttributeError, attr
  
  def getBaseObject(obj):
    while hasattr(obj, 'obj'):
      obj = obj.obj
    return obj
  getBaseObject = staticmethod(getBaseObject)
  
  def __eq__(self, obj):
    if not hasattr(obj, 'obj'):
      return 0
    return self.isSame(obj)
  
  def __hash__(self):
    return hash(id(self.getBaseObject(self)))
  
  def __nonzero__(self):
    return 1
  
  def __repr__(self):
    return '<class ' + self.getClassName() + ' at ' + hex(id(self)) + '>'
  
  def __str__(self):
    return '<class ' + self.getClassName() + ' at ' + hex(id(self)) + '>'
  
  def isSame(self, obj):
    '''
    Return true if and only if <code>obj</code> refers to the same underlying object as this object
    '''
    return id(self.getBaseObject(self)) == id(self.getBaseObject(obj))
  
  def getClassName(self):
    '''
    Return the type name of the underlying class object
    '''
    return 'ASE.Loader'
  
  def isInstanceOf(self, typeName):
    '''
    Return <code>true</code> if this object is an instance of the specified type
    - This routine will return <code>true</code> if and only if a cast to the specified type would succeed
    '''
    if len(self.classes) > 1:
      import ASE.Loader
      return ASE.Loader.Loader.createClass(self.classes[0].__module__[:-5]).isInstanceOf(typeName)
    elif len(self.classes) == 0 and not self.obj is None:
      return self.obj.isInstanceOf(typeName)
    elif typeName in ('ASE.BaseInterface', 'ASE.BaseClass', 'ASE.Loader'):
      return 1
    return 0
  
  def getLibraries():
    '''
    Retrieve the list of loaded libraries
    '''
    return implMod.getLibraries()
  getLibraries = staticmethod(getLibraries)
  
  def setLibraries(libraries):
    '''
    Set the list of loaded libraries
    '''
    return implMod.setLibraries(libraries)
  setLibraries = staticmethod(setLibraries)
  
  def addLibraries(libraries):
    '''
    Append the specified DLL to the beginning of the list of loaded libraries
    '''
    return implMod.addLibraries(libraries)
  addLibraries = staticmethod(addLibraries)
  
  def removeLibraries(libraries):
    '''
    Remove the specified DLL from the list of loaded libraries
    '''
    return implMod.removeLibraries(libraries)
  removeLibraries = staticmethod(removeLibraries)
  
  def createClass(className):
    '''
    Create an instance of the SIDL class
    - If the class constructor cannot be found in one of the already loaded libraries,
    then the method will search through the library search path
    - If the class constructor cannot be found, then return NULL
    - For templated classes, the SIDL class name is augmented, e.g. ASE.MyClass<int>
    '''
    mod = __import__(className)
    components = className.split('.')
    for comp in components[1:]:
      mod = getattr(mod, comp)
    return getattr(mod, components[-1])()
  createClass = staticmethod(createClass)
  
  def getClassMethodTable(className):
    '''
    Lookup a class IOR method table and return the associated pointer
    - If the class cannot be found in one of the already loaded libraries,
    then the method will search through the library search path
    - If the class does not exist, then return NULL
    '''
    return implMod.getClassMethodTable(className)
  getClassMethodTable = staticmethod(getClassMethodTable)
  
  def getSearchPath():
    '''
    Retrieve the search path
    '''
    return implMod.getSearchPath()
  getSearchPath = staticmethod(getSearchPath)
  
  def setSearchPath(uri):
    '''
    Set the search path
    '''
    return implMod.setSearchPath(uri)
  setSearchPath = staticmethod(setSearchPath)
